#!/usr/bin/env python

import os
import sys
import string

try:
	import pygtk
	#tell pyGTK, if possible, that we want GTKv2
	pygtk.require("2.0")
except:
	#Some distributions come with GTK2, but not pyGTK
	pass
try:
	import gtk
	import gtk.glade
	import gobject
except:
	print "You need to install pyGTK or GTKv2 ",
	print "or set your PYTHONPATH correctly."
	print "try: export PYTHONPATH=",
	print "/usr/local/lib/python2.2/site-packages/"
	sys.exit(1)


class TMainFrm:
	"""
	Main window of application
	"""

	def __init__(self):
		"""
		Class contructor, will run necessary methods to get everything
		ready.
		"""
		self.initializeProperties()
		self.loadGUI()
		self.connectSignals()
		self.loadImages()
		self.prepareList()
		self.doLoad()
		self.pushWidgetData()
		self.selectFirst()
		return

	def initializeProperties(self):
		"""
		Initialize class properties that will be needed later on
		"""
		self.gladefile = "system-config-boot.glade"
		self.windowname = "MainFrm"
		self.lastaction = ""
		return

	def loadGUI(self):
		"""
		Loads the glade file for creation of the GUI
		"""
		self.wTree = gtk.glade.XML (os.path.join(path,self.gladefile), self.windowname)
		return

	def connectSignals(self):
		"""
		All event handlers are connected here to the application
		"""
		dic = { "on_MainFrm_destroy" : gtk.main_quit,
				"on_serverinfo_destroy" : gtk.main_quit,
				"on_MoveUpBtn_clicked" : self.MoveUpBtn_clicked,
				"on_MoveDownBtn_clicked" : self.MoveDownBtn_clicked,
				"on_NewBtn_clicked" : self.NewBtn_clicked,
				"on_EditBtn_clicked" : self.EditBtn_clicked,
				"on_DeleteBtn_clicked" : self.DeleteBtn_clicked,
				"on_DefaultBtn_clicked" : self.DefaultBtn_clicked,
				"on_FallbackBtn_clicked" : self.FallbackBtn_clicked,
				"on_OkBtn_clicked" : self.OkBtn_clicked,
		        "on_CancelBtn_clicked" : self.CancelBtn_clicked }
		self.wTree.signal_autoconnect(dic)
		return

	def loadImages(self):
		"""
		Loads all images that will be used dynamically in the application
		"""
		self.pixbufDefault = gtk.gdk.pixbuf_new_from_file( os.path.join(path,"1.gif") )
		self.pixbufFallback = gtk.gdk.pixbuf_new_from_file( os.path.join(path,"2.gif") )
		return

	def prepareList(self):
		"""
		Assumes that the ListView already exists and assoicates a ListStore
		to it to handle the data
		"""
		# Create the model.
		self.view = self.wTree.get_widget("TreeList")
		self.model = gtk.ListStore(gobject.TYPE_STRING, gtk.gdk.Pixbuf, gtk.gdk.Pixbuf)		
		# Attach data view
		self.view.set_model(self.model)
		self.view.set_reorderable(True)
		self.setColumns()
		# Show list
		self.view.show()
		return

	def setColumns(self):
		"""
		Define the columns to be stored by the list
		"""
		column = gtk.TreeViewColumn("Title", gtk.CellRendererText(), text=0)
		column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		column.set_expand(True)
		column.set_resizable(True)
		self.view.append_column(column)
		column = gtk.TreeViewColumn("Default", gtk.CellRendererPixbuf(), pixbuf=1)
		column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
		column.set_fixed_width(75)
		self.view.append_column(column)
		column = gtk.TreeViewColumn("Fallback", gtk.CellRendererPixbuf(), pixbuf=2)
		column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
		column.set_fixed_width(75)
		self.view.append_column(column)
		return

	def doQuit(self):
		"""
		End the application gracefully
		"""
		gtk.main_quit()
		return

	def doLoad(self):
		"""
		Load the configuration file so that it may be parsed
		"""
		global grubconf
		grubconf.load(filepath)
		return

	def doBackup(self):
		"""
		Make a backup copy of the initial conf file
		"""
		os.rename(filepath, filepath+backup_suffix)
		return
	
	def doSave(self):
		"""
		Save the configuration file to disk
		"""
		grubconf.save(filepath)
		return

	def pushWidgetData(self):
		"""
		Assigns data from objects to the associated widgets
		"""
		global grubconf
		count = 0
		self.indexDefault = grubconf.default
		self.indexFallback = grubconf.fallback

		checkbox = self.wTree.get_widget("HideMenuChkBx")
		spinbox = self.wTree.get_widget("TimeoutSpin")

		checkbox.set_active(grubconf.hiddenmenu)
		spinbox.set_value(grubconf.timeout)
		
		for entry in grubconf.entries:
			count = count + 1
			if count == grubconf.default+1:
				iter = self.model.append( (entry.title, self.pixbufDefault, None) )
			elif count == grubconf.fallback+1:
				iter = self.model.append( (entry.title, None, self.pixbufFallback) )
			else:
				iter = self.model.append( (entry.title, None , None) )

		return

	def pullWidgetData(self):
		"""
		Gets data from the associated widgets to the objects
		"""
		global grubconf
		grubconf.default = self.indexDefault
		grubconf.fallback = self.indexFallback

		checkbox = self.wTree.get_widget("HideMenuChkBx")
		spinbox = self.wTree.get_widget("TimeoutSpin")

		grubconf.hiddenmenu = checkbox.get_active()
		grubconf.timeout = spinbox.get_value()
		grubconf.timeout = int(grubconf.timeout)

		return

	def selectFirst(self):
		"""
		Ensures that the first item in the list is selected.
		This prevents a lot pof potential problems with not having
		anything selected.
		"""
		view_select = self.view.get_selection()
		view_select.select_path( (0,) )
		return

	def updateSelection(self):
		"""
		This function is called by the Entry Dialog just before it
		destroys itself if the user wanted changes made. It hadles
		updating of the list:  updates title if just editing was done,
		adds a new entry to the list if a new one was created.
		"""
		global grubconf, newentry
		# Get selection
		view_select = self.view.get_selection()
		model, iter = view_select.get_selected()

		if self.lastaction == "edit":
			if iter != None:
				index =  model.get_path(iter)[0]
				self.model.set_value(iter, 0, grubconf.entries[index].title)

		if self.lastaction == "new":
			if iter == None:
				if not entry.title: newentry.title = "Untitled " + str( len(grubconf.entries)+1 )
				grubconf.entries.append(newentry)
				self.model.append( (newentry.title, None, None) )
			else:
				index =  model.get_path(iter)[0]
				if not newentry.title: newentry.title = "Untitled " + str( index+1+1 )
				
				grubconf.entries.insert(index+1, newentry)
				self.model.insert_after(iter, (newentry.title, None, None) )
				if self.indexDefault < index:
					self.indexDefault = self.indexDefault + 1
				if self.indexFallback < index:
					self.indexFallback = self.indexFallback + 1
		return

	def MoveUpBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'Up' push button
		This will swap the selected row in the list up if there is a
		row above it.
		"""
		global grubconf
		# Get selection
		view_select = self.view.get_selection()
		model, iter = view_select.get_selected()
		if iter != None:
			index = model.get_path(iter)[0]
			prev = model.get_iter( (index-1, ) )
			if prev != None:
				if self.indexDefault == index:
					self.indexDefault = self.indexDefault - 1
				if self.indexFallback == index:
					self.indexFallback = self.indexFallback - 1
				grubconf.entries[index-1], grubconf.entries[index] = grubconf.entries[index], grubconf.entries[index-1]
				model.swap(iter, prev)
		return

	def MoveDownBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'Down' push button
		This will swap the selected row in the list down if there is a
		row below it.
		"""
		# Get selection
		view_select = self.view.get_selection()
		model, iter = view_select.get_selected()
		if iter != None:
			next = model.iter_next(iter)
			if next != None:
				index = model.get_path(iter)[0]
				if self.indexDefault == index:
					self.indexDefault = self.indexDefault + 1
				if self.indexFallback == index:
					self.indexFallback = self.indexFallback + 1
				grubconf.entries[index], grubconf.entries[index+1] = grubconf.entries[index+1], grubconf.entries[index]
				model.swap(iter, next)
		return

	def NewBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'New' push button
		This will create a empty temporary entry and pass it to the entry
		editing dialog. It also notes that is has been executed as an action
		so that the updating knows what to do.
		"""
		global newentry
		self.lastaction = "new"
		newentry = TGrubConfEntry()
		EntryDiag = TEntryDiag(self, newentry)
		return

	def EditBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'Up' push button
		This will swap the selected row in the list up if there is a
		row above it.
		"""
		global grubconf
		# Get selection
		view_select = self.view.get_selection()
		model, iter = view_select.get_selected()
		if iter != None:
			self.lastaction = "edit"
			index =  model.get_path(iter)[0]
			EntryDiag = TEntryDiag(self, grubconf.entries[index])
		return

	def DeleteBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'Delete' push button
		This will make necessary changes to the `default` and `fallback`
		values, then remove the entry from the object, then finally
		remove it from the list view.
		"""
		global grubconf
		# Get selection
		view_select = self.view.get_selection()
		model, iter = view_select.get_selected()
		if iter != None:
			if model.iter_n_children( model.iter_parent(iter) ) > 1:
				# if the default value is below the selection
				if self.indexDefault > model.get_path(iter)[0]:
					self.indexDefault = self.indexDefault - 1
				# if the fallback value is below the selection
				if self.indexFallback > model.get_path(iter)[0]:
					self.indexFallback = self.indexFallback - 1
				# if the selection is the default 
				if self.indexDefault == model.get_path(iter)[0]:
					# then set the following row as the default
					iter = model.iter_next(iter)
					if iter != None:
						model.set(iter, 1, self.pixbufDefault, 2, None)
					else:
						# else just sent the row above as the default
						iter = model.get_iter( (model.get_path(iter)[0]-1, ) )
						model.set(iter, 1, self.pixbufDefault, 2, None)
						# reflect the change in the default value
						self.indexDefault = self.indexDefault - 1
				# remove entry in object first
				del grubconf.entries[ model.get_path(iter)[0] ]
				# then remove from list
				model.remove(iter)
		return

	def DefaultBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'Default' push button
		This will set the selected row to be the default entry
		"""
		# Get selection
		view_select = self.view.get_selection()
		model, iter = view_select.get_selected()
		if iter != None:
			if self.indexDefault != model.get_path(iter)[0]:
				if self.indexFallback != model.get_path(iter)[0]:
					model.set(iter, 1, self.pixbufDefault, 2, None)
					if self.indexDefault > -1:
						old_iter = model.get_iter( (self.indexDefault, ) )
						model.set(old_iter, 1, None, 2, None)
					self.indexDefault = model.get_path(iter)[0]
		return

	def FallbackBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'Default' push button
		This will set the selected row to be the fallback entry
		"""
		# Get selection
		view_select = self.view.get_selection()
		model, iter = view_select.get_selected()
		if iter != None:
			if self.indexFallback == model.get_path(iter)[0]:
				model.set(iter, 1, None, 2, None)
				self.indexFallback = -1
			elif self.indexDefault != model.get_path(iter)[0]:
				model.set(iter, 1, None, 2, self.pixbufFallback)
				if self.indexFallback > -1:
					old_iter = model.get_iter( (self.indexFallback, ) )
					model.set(old_iter, 1, None, 2, None)
				self.indexFallback = model.get_path(iter)[0]
		return

	def OkBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'Ok' push button
		This will gather the data from the widgets, possibly do a backup
		of the original conf file, save the current conf data to file,
		then finally quit the application
		"""
		self.pullWidgetData()
		if backup: self.doBackup()
		self.doSave()
		self.doQuit()
		return
	
	def CancelBtn_clicked(self,sender):
		"""
		Event Handler: Responds to clicking of the 'Cancel' push button
		This will simpyly quit the application.
		"""
		self.doQuit()
		return


class TEntryDiag:
	"""
	This dialog handles editing of the boot entries
	"""

	def __init__(self, parent, entry):
		"""
		Set the class properties, load the GUI, and connect the
		event handlers. Then load the images that may be used,
		prepare the list view and put the data into the widgets.
		"""
		self.gladefile = "system-config-boot.glade"
		self.windowname = "EntryDiag"
		self.parent = parent
		self.data = entry
		self.wTree = gtk.glade.XML(os.path.join(path,self.gladefile), self.windowname)
		
		dic = { "on_EntryDiag_response" : self.response,
		        "on_AddModuleBtn_clicked" : self.AddModuleBtn_clicked,
				"on_RemoveModuleBtn_clicked" :self.RemoveModuleBtn_clicked }
		self.wTree.signal_autoconnect(dic)
		self.loadImages()
		self.prepareList()
		self.pushWidgetData()
		return

	def __repr__(self):
		text = self.title
		return text

	def loadImages(self):
		"""
		Load the images to be used
		"""
		self.pixbufUnzip = gtk.gdk.pixbuf_new_from_file( os.path.join(path,"unzip.gif") )
		self.pixbufNoUnzip = gtk.gdk.pixbuf_new_from_file( os.path.join(path,"nounzip.gif") )
		return

	def prepareList(self):
		"""
		Setup the list view: attach a list store, and insert the columns.
		"""
		# Create the model.
		self.model = gtk.ListStore(gtk.gdk.Pixbuf, gobject.TYPE_STRING)
		
		# Create data view
		self.view = self.wTree.get_widget("ModulesList")
		self.view.set_model(self.model)

		# Add columns
		column = gtk.TreeViewColumn("Unzip", gtk.CellRendererPixbuf(), pixbuf=0)
		column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
		column.set_fixed_width(50)
		self.view.append_column(column)
		column = gtk.TreeViewColumn("File", gtk.CellRendererText(), text=1)
		column.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
		column.set_expand(True)
		column.set_resizable(True)
		self.view.append_column(column)
		return

	def pushWidgetData(self):
		"""
		Data from the object is assigned to the widgets. All data is assumed
		valid.
		"""
		self.wTree.get_widget("TitleEdit").set_text(self.data.title)
		self.wTree.get_widget("RootEdit").set_text(self.data.root)
		self.wTree.get_widget("KernelEdit").set_text(self.data.kernel)
		self.wTree.get_widget("KernelParamEdit").set_text(self.data.kernel_param)
		self.wTree.get_widget("RamdiskEdit").set_text(self.data.ramdiskimg)
		self.wTree.get_widget("ChainloaderEdit").set_text(self.data.chainloader)

		self.wTree.get_widget("NoMountToggle").set_active(self.data.rootnoverify)
		self.wTree.get_widget("MakeRootActiveToggle").set_active(self.data.makeactive)
		self.wTree.get_widget("ProtectToggle").set_active(self.data.lock)
		self.wTree.get_widget("KernelBootTypeRadio").set_active(self.data.boottype == "kernel")
		self.wTree.get_widget("ChainloaderBootTypeRadio").set_active(self.data.boottype == "chainloader")
		self.wTree.get_widget("ForceBtn").set_active(self.data.chainloader_force)

		if self.data.pause:
			self.wTree.get_widget("YesPauseRadio").set_active(True)
			self.wTree.get_widget("PauseMsgEdit").set_text(self.data.pause_msg)
		else:
			self.wTree.get_widget("NoPauseRadio").set_active(True)

		for module in self.data.modules:
			if module["unzip"]:
				iter = self.model.append( (self.pixbufUnzip, module["file"]) )
			else:
				iter = self.model.append( (self.pixbufNoUnzip, module["file"]) )

		return

	def pullWidgetData(self):
		"""
		The object is populated with data from the widgets. All data is assumed
		valid, and no trimming is done.
		"""
		self.data.title = self.wTree.get_widget("TitleEdit").get_text()
		self.data.root = self.wTree.get_widget("RootEdit").get_text()
		self.data.kernel = self.wTree.get_widget("KernelEdit").get_text()
		self.data.kernel_param = self.wTree.get_widget("KernelParamEdit").get_text()
		self.data.ramdiskimg = self.wTree.get_widget("RamdiskEdit").get_text()
		self.data.chainloader = self.wTree.get_widget("ChainloaderEdit").get_text()

		self.data.rootnoverify = self.wTree.get_widget("NoMountToggle").get_active()
		self.data.makeactive = self.wTree.get_widget("MakeRootActiveToggle").get_active()
		self.data.lock = self.wTree.get_widget("ProtectToggle").get_active()
		self.data.chainloader_force = self.wTree.get_widget("ForceBtn").get_active()

		if self.wTree.get_widget("KernelBootTypeRadio").get_active():
			self.data.boottype = "kernel"
		if self.wTree.get_widget("ChainloaderBootTypeRadio").get_active():
			self.data.boottype == "chainloader"

		self.data.pause = self.wTree.get_widget("YesPauseRadio").get_active()
		if self.data.pause:
			self.data.pause_msg = self.wTree.get_widget("PauseMsgEdit").get_text()

		self.data.modules = []
		iter = self.model.get_iter_first()
		while iter != None:
			self.data.modules.append( {"unzip": self.model.get_value(iter, 0) == self.pixbufUnzip, "file": self.model.get_value(iter, 1) } )
			iter = self.model.iter_next(iter)

		return

	def response(self, dialog, response):
		# OK button clicked
		if response == gtk.RESPONSE_OK:
			self.pullWidgetData()
			self.parent.updateSelection()
			dialog.destroy()
		# CANCEL button clicked
		elif response == gtk.RESPONSE_CANCEL:
			dialog.destroy()
			return
		# dialog closed by other means, such as the 'X' button
		elif response == gtk.RESPONSE_DELETE_EVENT:
			dialog.destroy()
			return
		else:
			print response
		return

	def AddModuleBtn_clicked(self, sender):
		"""
		Event Handler: Responds to clicking of the 'Add' push button
		This will create a dialog to add a module
		"""
		ModuleDiag = TModuleDiag(self)
		return

	def AddModuleBtn_response(self, sender):
		"""
		This will be called by the dialog for adding a module once its
		OK button is clicked
		"""
		if sender.file.get_text().strip():
			if sender.unzip.get_active():
				iter = self.model.append( (self.pixbufUnzip, sender.file.get_text().strip()) )
			else:
				iter = self.model.append( (self.pixbufNoUnzip, sender.file.get_text().strip()) )
		return

	def RemoveModuleBtn_clicked(self, sender):
		"""
		Event Handler: Responds to clicking of the 'Delete' push button
		This will swap the selected row in the list up if there is a
		row above it.
		"""
		view_select = self.view.get_selection()
		model, iter = view_select.get_selected()
		if iter != None:
			model.remove(iter)
		return


class TModuleDiag:
	def __init__(self, parent):
		"""
		Set the class properties, load the GUI, attach the event handlers,
		and grab the only two widgets as class properties.
		"""
		self.parent = parent
		self.gladefile = "system-config-boot.glade"
		self.windowname = "ModuleDiag"
		self.wTree = gtk.glade.XML(os.path.join(path,self.gladefile), self.windowname)
		
		dic = { "on_ModuleDiag_response" : self.response }
		self.wTree.signal_autoconnect(dic)

		self.file = self.wTree.get_widget("FileEdit")
		self.unzip = self.wTree.get_widget("UnzipBtn")
		return

	def response(self, dialog, response):
		"""
		Respond to an even from the dialog
		"""
		if response == gtk.RESPONSE_OK:
			self.parent.AddModuleBtn_response(self)
			dialog.destroy()
		elif response == gtk.RESPONSE_CANCEL:
			dialog.destroy()
		elif response == gtk.RESPONSE_DELETE_EVENT:
			dialog.destroy()
		else:
			print response
		return


class TGrubConf:
	"""
	This class stores all the contents of a grub.conf file, evem what it
	cannot understand.
	"""
	def __init__(self):
		"""
		Intialize all the class properties
		"""
		self.initializeProperties()
		return

	def initializeProperties(self):
		"""
		Give all the data that will be used rational intial values
		"""
		self.pastTitle = False
		self.default = -1
		self.fallback = -1
		self.hiddenmenu = False
		self.timeout = 0
		self.entries = []
		self.unrecognized = []
		return

	def load(self, path):
		"""
		Parse the data out of the file, igoring all comments, and striping away
		all whitespace and line ends.
		*No error handling here yet*
		"""
		pastTitle = False
		conffile = open(path, 'r')
		for line in conffile.readlines():
			line = line.strip()
			words = line.split()
			if len(words) == 0: continue
			if words[0] == '#': continue
			if words[0][0] == '#': continue
			self.readline(line)
		conffile.close()
		return

	def save(self, path):
		"""
		Saves a properly formatted grub.conf to the given path.
		*No error handling here yet*
		"""
		output = open(path, 'w')
		self.writeln(output, "# grub.conf generated by system-config-boot")
		self.writeln(output, "#")
		self.writeln(output, "#")
		self.write(output)
		output.close()
		return

	def readline(self,line):
		"""
		Parse the conf line by line
		"""
		words = line.split()

		if words[0] == "title":
			del words[0]
			self.pastTitle = True
			self.entries.append( TGrubConfEntry( " ".join(words) ) )
		else:
			if self.pastTitle:
				self.entries[ len(self.entries)-1 ].readline(line)
			else:
				if words[0] == "hiddenmenu":
					self.hiddenmenu = True
					return
				elif line.find('=') > -1:
					value = line.split('=', 1)
				else:
					value = line.split(' ', 1)
				if len(value) > 1:
					if value[0] == "default":
						if value[1].isdigit():
							self.default = int(value[1])
						else:
							print "Invalid `default` value"
					elif value[0] == "fallback":
						if value[1].isdigit():
							self.fallback = int(value[1])
						else:
							print "Invalid `fallback` value"
					elif value[0] == "timeout":
						if value[1].isdigit():
							self.timeout =  int(value[1])
						else:
							print "Invalid `timeout` value"
					else:
						self.unrecognized.append(line)
						print "Unrecognized ->", line
		return

	def write(self, output):
		"""
		Write the conf contents to an output stream
		"""
		global grubconf
		self.writeln(output, "default=" + str(self.default))

		if self.fallback > -1:
			self.writeln(output, "fallback=" + str(self.fallback))

		self.writeln(output, "timeout=" + str(self.timeout))

		if self.hiddenmenu:
			self.writeln(output, "hiddenmenu")

		for line in self.unrecognized:
			self.writeln(output, line)

		for entry in self.entries:
			self.writeln(output)
			entry.write(output)
		return

	def writeln(self, output, line=""):
		"""
		Simple function to include a line break at the end of each line
		"""
		output.write(line + os.linesep)
		return

class TGrubConfEntry:
	"""
	Stores a boot entry and its data
	"""
	def __init__(self, title=""):
		"""
		Initialize class properties
		"""
		self.initializeProperties()
		self.title = title
		return

	def initializeProperties(self):
		"""
		Set data to be used to rational initial values
		"""
		self.title = ""
		self.root = ""
		self.rootnoverify = False
		self.makeactive = False
		self.lock = False
		self.pause = False
		self.boottype = ""
		self.pause_msg = ""
		self.kernel = ""
		self.kernel_param = ""
		self.ramdiskimg = ""
		self.chainloader = ""
		self.chainloader_force = False
		self.modules = []
		self.unrecognized = []
		return

	def readline(self, line):
		"""
		Parse each line sent to it for data. Does some whitespace cleaning
		"""
		command = line.split(' ', 1)
		if command[0] == "kernel":
			command = line.split(' ', 2)
			self.kernel = command[1].strip()
			if len(command) > 2:
				self.kernel_param = command[2].strip()
			self.boottype = "kernel"
		elif command[0] == "root":
			self.root = command[1].strip()
		elif command[0] == "rootnoverify":
			self.root = command[1].strip()
			self.rootnoverify = True
		elif command[0] == "initrd":
			self.ramdiskimg = command[1].strip()
		elif command[0] == "chainloader":
			command = line.split(' ', 2)
			if len(command) > 2:
				if command[1].strip() == "--force":
					self.chainloader_force = False
					self.chainloader = command[2].strip()
				else:
					print "Unrecognized parameter passed to chainloader command"
			else:
				self.chainloader = command[1].strip()
			self.boottype = "chainloader"
		elif command[0] == "makeactive":
			self.makeactive = True
		elif command[0] == "lock":
			self.lock = True
		elif command[0] == "pause":
			self.pause = True
			self.pause_msg = command[1].strip()
		elif command[0] == "module":
			self.modules.append( {"unzip": True, "file": command[1].strip()} )
		elif command[0] == "modulenounzip":
			self.modules.append( {"unzip": False, "file": command[1].strip()} )
		else:
			self.unrecognized.append(line)
			print "Unrecognized ->", line
		return

	def write(self, output):
		"""
		Handles writing out the data in the entry to an output stream
		"""
		self.writeln(output, "title " + self.title)

		if self.root:
			if self.rootnoverify:
				self.writeln(output, "\t" + "rootnoverify " + self.root)
			else:
				self.writeln(output, "\t" + "root " + self.root)

		if self.makeactive: self.writeln(output, "\t" + "makeactive")
		if self.lock: self.writeln(output, "\t" + "lock")

		if self.pause:
			if self.pause_msg:
				self.writeln(output, "\t" + "lock " + self.pause_msg)
			else:
				self.writeln(output, "\t" + "pause")

		if self.boottype == "kernel":
			if self.kernel:
				if self.kernel_param:
					self.writeln(output, "\t" + "kernel " + self.kernel + " " + self.kernel_param)
				else:
					self.writeln(output, "\t" + "kernel " + self.kernel)
			if self.ramdiskimg:
				self.writeln(output, "\t" + "initrd " + self.ramdiskimg)
			for module in self.modules:
				if module["unzip"]:
					self.writeln(output, "\t" + "module " + module["file"])
				else:
					self.writeln(output, "\t" + "modulenounzip " + module["file"])

		if self.boottype == "chainloader":
			if self.chainloader:
				if self.chainloader_force:
					self.writeln(output, "\t" + "chainloader --force " + self.chainloader)
				else:
					self.writeln(output, "\t" + "chainloader " + self.chainloader)

		for line in self.unrecognized:
			self.writeln(output, line)

		return

	def writeln(self, output, line):
		"""
		Simple function to add a line break at the end of the string
		"""
		output.write(line + os.linesep)
		return

def main():
	global filepath
	if len(sys.argv) > 1:
		filepath = sys.argv[1]
	if not os.path.isfile(filepath):
		print "The path '%s' was not to a valid file!" % (filepath)
	else:
		MainFrm = TMainFrm()
		gtk.main()
	return


# backups only occur if this is true
backup = True
backup_suffix = ".bak"
# path where the conf file will be at
path = sys.path[1]
# expected patch to conf file
if os.geteuid() == 0:
	filepath = "/boot/grub/grub.conf"
else:
	filepath = "grub.conf"
# object to store all conf data
grubconf = TGrubConf()
# temporary entry
newentry = None


if __name__ == "__main__":
	main()

############################################################################
#    Copyright (C) 2006 by Arthur Pemberton                                #
#    www.pembo13.com
#    pembo13@gmail.com                                                     #
#                                                                          #
#    This program is free software; you can redistribute it and#or modify  #
#    it under the terms of the GNU General Public License as published by  #
#    the Free Software Foundation; either version 2 of the License, or     #
#    (at your option) any later version.                                   #
#                                                                          #
#    This program is distributed in the hope that it will be useful,       #
#    but WITHOUT ANY WARRANTY; without even the implied warranty of        #
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
#    GNU General Public License for more details.                          #
#                                                                          #
#    You should have received a copy of the GNU General Public License     #
#    along with this program; if not, write to the                         #
#    Free Software Foundation, Inc.,                                       #
#    59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################